import React, { useState, useEffect, useRef } from "react";

const clamp = ({ left, top, width, height }) => {
  if (left < 0) {
    left = 0;
  } else if (left + width > window.innerWidth) {
    left = window.innerWidth - width;
  }

  // TODO clamp top based on a prop

  return { left, top };
};

function Dropdown({ children, dropdown }) {
  // TODO add preferred place support
  const [position, setPosition] = useState({ left: 0, top: 0 });
  const [isDropdownShown, setIsDropdownShown] = useState(false);
  const refContainer = useRef(null);
  const refDropdown = useRef(null);

  function calculatePosition(e) {
    const { left, top, /*width,*/ height } =
      refContainer && refContainer.current.getBoundingClientRect();

    const clamped = clamp({
      left,
      top,
      width: refDropdown && refDropdown.current.getBoundingClientRect().width,
      height: refDropdown && refDropdown.current.getBoundingClientRect().height,
    });
    setPosition({ left: clamped.left, top: clamped.top + height });
  }

  useEffect(() => {
    // TODO add throttling
    window.addEventListener("scroll", calculatePosition);
    return () => window.removeEventListener("scroll", calculatePosition);
  }, []);

  return (
    <div
      onMouseLeave={() => setIsDropdownShown(false)}
      onMouseEnter={() => {
        calculatePosition();
        setIsDropdownShown(true);
      }}
    >
      <div ref={refContainer}>{children}</div>
      <div
        ref={refDropdown}
        style={{
          visibility: isDropdownShown ? "visible" : "hidden",
          position: "fixed",
          top: position.top,
          left: position.left,
        }}
      >
        {dropdown}
      </div>
    </div>
  );
}

export default Dropdown;
